import Foundation
import ProjectDescription
import os.log

public enum TargetType {
  case framework
  case tests
}

extension Target {
  public static func makeAppTargets(
    name: String,
    dependencies: [Targets] = [],
    externals: [Dependency] = [],
    testDependencies: [String] = []
  ) -> [Target] {

    let bundleId = makeBundleId(with: name)

    let infoPlist = [String: InfoPlist.Value]()
    
    var targetDependencies: [TargetDependency] = dependencies.map { .target(name: $0.rawValue) }
    targetDependencies.append(contentsOf: externals.map { .external(name: $0.rawValue) })

    return [
      Target(
        name: name,
        platform: .iOS,
        product: .app,
        bundleId: bundleId,
        deploymentTarget: .iOS(
          targetVersion: "14.5",
          devices: .iphone
        ),
        infoPlist: .extendingDefault(with: infoPlist),
        sources: ["Targets/\(name)/Sources/**"],
        dependencies: targetDependencies,
        settings: .makeAppSettings(bundleId)
      )
    ]
  }

  public static func makeFramework(
    target: Targets,
    targets: [Targets] = [],
    externals: [Dependency] = [],
    additional: [TargetDependency] = [],
    types: Set<TargetType> = [.framework]
  )
    -> [Target]
  {

    let name = target.rawValue
    let bundleId = makeBundleId(with: name)

    let infoPlist = [String: InfoPlist.Value]()

    // Target dependencies
    var targetDependencies: [TargetDependency] = targets.map { .target(name: $0.rawValue) }
    targetDependencies.append(contentsOf: externals.map { .external(name: $0.rawValue) })
    targetDependencies += additional

    // Targets
    var projectTargets: [Target] = []
    if types.contains(.framework) {
      projectTargets.append(
        Target(
          name: name,
          platform: .iOS,
          product: .framework,
          bundleId: bundleId,
          deploymentTarget: .iOS(
            targetVersion: "14.5",
            devices: .iphone
          ),
          infoPlist: .extendingDefault(with: infoPlist),
          sources: ["Targets/\(name)/Sources/**"],
          dependencies: targetDependencies,
          settings: Settings.makeTargetSettings()))
    }

    return projectTargets
  }

  fileprivate static func makeBundleId(with name: String) -> String {
    "com.artem.krachulov.\(name.lowercased())"
  }
}
